package client.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketTimeoutException;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.ListSelectionModel;

import server.data.UtilMethods;
import auxdata.ConInfo;


public class ClientWindow extends JFrame {	

	// CUSTOM PRIVATES

	private File chosenFile = null;
	private InetAddress serverAddress = null;
	private int selectedFile;

	//

	private static final long serialVersionUID = 1L;

	private JPanel jContentPane = null;

	private JButton chooseFileButton = null;

	private Dimension windowSize = new Dimension(600,400);

	private JButton connectButton = null;

	private JLabel conStatTitle = null;

	private JLabel conStat = null;

	private JLabel mainText = null;

	private JButton uploadButton = null;

	private JProgressBar uploadProgressBar = null;

	private JPanel topPanel = null;

	private JPanel mainPanel = null;

	private JPanel bottomPanel = null;

	private JList fileList = null;

	private DefaultListModel fileListModel = null;
	private JButton downloadButton = null;

	/**
	 * This is the default constructor
	 */
	public ClientWindow() {
		super();
		initialize();

	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(601,400);
		this.setLocation(new Point(200, 200));
		this.setName("mainFrame");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setMinimumSize(windowSize);
		this.setPreferredSize(windowSize);
		this.setContentPane(getJContentPane());
		this.setTitle("Backup 1.0");
		this.setVisible(true);
		this.pack();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.insets = new Insets(0, 10, 10, 10);
			gridBagConstraints8.gridy = 2;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.fill = GridBagConstraints.BOTH;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.weighty = 1.0;
			gridBagConstraints7.insets = new Insets(5, 10, 5, 10);
			gridBagConstraints7.gridy = 1;
			GridBagConstraints gridBagConstraints61 = new GridBagConstraints();
			gridBagConstraints61.gridx = 0;
			gridBagConstraints61.gridwidth = 1;
			gridBagConstraints61.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints61.weightx = 1.0;
			gridBagConstraints61.insets = new Insets(10, 10, 0, 10);
			gridBagConstraints61.gridy = 0;
			mainText = new JLabel();
			mainText.setText("");
			conStat = new JLabel();
			conStat.setText("Not connected...");
			conStatTitle = new JLabel();
			conStatTitle.setText("Connection status:");
			jContentPane = new JPanel();
			jContentPane.setLayout(new GridBagLayout());
			jContentPane.add(getTopPanel(), gridBagConstraints61);
			jContentPane.add(getMainPanel(), gridBagConstraints7);
			jContentPane.add(getBottomPanel(), gridBagConstraints8);
		}
		return jContentPane;
	}

	/**
	 * This method initializes chooseFileButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getChooseFileButton() {
		if (chooseFileButton == null) {
			chooseFileButton = new JButton();
			chooseFileButton.setEnabled(false);
			chooseFileButton.setText("Choose file");
			chooseFileButton.setActionCommand("filechosen");
			chooseFileButton.addActionListener(new ActionHandler());
		}
		return chooseFileButton;
	}

	/**
	 * This method initializes connectButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getConnectButton() {
		if (connectButton == null) {
			connectButton = new JButton();
			connectButton.setText("Connect");
			connectButton.setActionCommand("connect");
			connectButton.addActionListener(new ActionHandler());
		}
		return connectButton;
	}

	// PRIVATE CLASSES

	private class ActionHandler implements ActionListener {

		public void actionPerformed(ActionEvent arg0) {

			String ac = arg0.getActionCommand();

			if (ac.equals("connect")) {
				try {
					DatagramSocket clientSocket = new DatagramSocket();
					InetAddress group = InetAddress.getByName(ConInfo.MULTICASTGROUP);			
					String requestMsg = new String(ConInfo.CLIENTRQUPLOAD);
					DatagramPacket sendPacket = 
						new DatagramPacket(requestMsg.getBytes(), requestMsg.getBytes().length, group, ConInfo.MULTICASTPORT);
					clientSocket.send(sendPacket);

					byte[] rcvbuf = new byte[8];
					DatagramPacket packet = new DatagramPacket(rcvbuf, rcvbuf.length);
					clientSocket.setSoTimeout(ConInfo.UDPANSWERTIMEOUT); // Stop blocking on receive() after 3 seconds
					clientSocket.receive(packet);

					String returnMsg = new String(packet.getData(), 0, packet.getLength());

					if (returnMsg.equals(ConInfo.ACK)) {
						conStat.setText("Server available at: " + packet.getAddress());
						serverAddress = packet.getAddress();
						mainText.setText("Choose file.");

						getChooseFileButton().setEnabled(true);
					}

				} catch (SocketTimeoutException e) {
					conStat.setText("No servers available!");
				} catch (SocketException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} else if (ac == "filechosen") {
				JFileChooser fc = new JFileChooser();
				fc.showOpenDialog(jContentPane.getTopLevelAncestor());

				chosenFile = fc.getSelectedFile();

				mainText.setText("<html>File chosen: <br><br><i>"
						+ fc.getSelectedFile() + "<br>" 
						+ fc.getSelectedFile().length() + " Bytes"
						+ "</i></html>");

				uploadButton.setEnabled(true);

			} else if (ac == "upload") {
				try {

					InputStream is;
					OutputStream os;
					byte[] returnBuf = new byte[32];
					int isBytesRead;
					int fisBytesRead;
					String returnMsg;

					Socket uploadSocket = new Socket(serverAddress, ConInfo.TCPLISTENPORT);
					is = uploadSocket.getInputStream();
					BufferedInputStream bis = new BufferedInputStream(is);
					os = uploadSocket.getOutputStream();

					// Send server the size of the file to be uploaded
					// Ugly but works. chosenFile.length() must be < max size of int.
					int fileSize = Integer.parseInt(String.valueOf(chosenFile.length()));
					byte[] BytedFileSize = UtilMethods.intToBytes(fileSize); 
					os.write(BytedFileSize); // Writes exactly 4 bytes

					// Wait for server to accept filesize
					isBytesRead = bis.read(returnBuf);
					returnMsg = new String(returnBuf, 0, isBytesRead);

					// Send the file if filesize is accepted
					if (returnMsg.equals(ConInfo.FILESIZERECEIVEDOK)) {
						int fSize = Integer.parseInt(String.valueOf(chosenFile.length()));
						byte[] fileBuf = new byte[fSize];
						FileInputStream fis = new FileInputStream(chosenFile);
						fisBytesRead = fis.read(fileBuf);
						// Error checking here would be an good idea
						os.write(fileBuf);

						// wait for a return ticket number
						byte[] BytedTicketNumber = new byte[4];
						bis.read(BytedTicketNumber);
						int ticketNumber = UtilMethods.bytesToInt(BytedTicketNumber, 0);

						// add the ticket number to the file list

						getFileListModel().addElement(new String("id#" + ticketNumber + ": " + chosenFile.getName()));

						os.close();
					} else {
						// Should return an NAK here
					}


				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			} else if (ac == "download") {
				try {
					
					JFileChooser save = new JFileChooser("Save as...");
					save.showSaveDialog(jContentPane.getTopLevelAncestor());
					chosenFile = save.getSelectedFile();

					// multicast to find out who has the file
					DatagramSocket clientSocket = new DatagramSocket();
					InetAddress group = InetAddress.getByName(ConInfo.MULTICASTGROUP);
					DatagramPacket sendPacket = 
						new DatagramPacket(
								UtilMethods.intToBytes(selectedFile), UtilMethods.intToBytes(selectedFile).length,
								group, ConInfo.MULTICASTPORT);
					clientSocket.send(sendPacket);

					// wait for reply
					byte[] protocolBuffer = new byte[8];
					DatagramPacket packet = new DatagramPacket(protocolBuffer, protocolBuffer.length);
					clientSocket.setSoTimeout(ConInfo.UDPANSWERTIMEOUT); // Stop blocking on receive() after 3 seconds
					clientSocket.receive(packet);
					
					// save host info
					
					InetAddress hostWithfile = packet.getAddress();
					
					// establish connection and request the file
					
					InputStream is;
					BufferedInputStream bis;
					OutputStream os;
					Socket fileSocket;
					
					int isBytesRead;
					int fisBytesRead;
					String returnMsg;

					fileSocket = new Socket(serverAddress, ConInfo.TCPLISTENPORT);
					is = fileSocket.getInputStream();
					bis = new BufferedInputStream(is);
					os = fileSocket.getOutputStream();

					// Send server the file id to be downloaded
					byte[] BytedTicketNumber = UtilMethods.intToBytes(selectedFile); 
					os.write(selectedFile); // Writes exactly 4 bytes

					byte[] file = new byte[20000000];
					
					// get the file
					bis.read(file);					
										
					// create the new file
					FileOutputStream fos = new FileOutputStream(chosenFile);
					fos.write(file);
					
					mainText.setText("<html>File saved: <br><br><i>"
							+ save.getSelectedFile() + "<br>" 
							+ save.getSelectedFile().length() + " Bytes"
							+ "</i></html>");
					
					fos.close();
					
				} catch (FileNotFoundException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}


			}

		}

	}


	/**
	 * This method initializes uploadButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getUploadButton() {
		if (uploadButton == null) {
			uploadButton = new JButton();
			uploadButton.setText("Upload");
			uploadButton.setEnabled(false);
			uploadButton.setActionCommand("upload");
			uploadButton.addActionListener(new ActionHandler());
		}
		return uploadButton;
	}

	/**
	 * This method initializes uploadProgressBar	
	 * 	
	 * @return javax.swing.JProgressBar	
	 */
	private JProgressBar getUploadProgressBar() {
		if (uploadProgressBar == null) {
			uploadProgressBar = new JProgressBar();
			uploadProgressBar.setPreferredSize(new Dimension(150,25));
		}
		return uploadProgressBar;
	}

	/**
	 * This method initializes topPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getTopPanel() {
		if (topPanel == null) {
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.anchor = GridBagConstraints.NORTHWEST;
			gridBagConstraints6.insets = new Insets(5, 40, 5, 0);
			gridBagConstraints6.gridwidth = 3;
			gridBagConstraints6.gridx = -1;
			gridBagConstraints6.gridy = -1;
			gridBagConstraints6.weighty = 0.0;
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.fill = GridBagConstraints.HORIZONTAL;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.anchor = GridBagConstraints.NORTHWEST;
			gridBagConstraints3.gridwidth = 1;
			gridBagConstraints3.gridx = -1;
			gridBagConstraints3.gridy = -1;
			gridBagConstraints3.weightx = 0.0;
			gridBagConstraints3.insets = new Insets(5, 20, 5, 0);
			topPanel = new JPanel();
			topPanel.setLayout(new GridBagLayout());
			topPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
			topPanel.add(conStatTitle, gridBagConstraints3);
			topPanel.add(conStat, gridBagConstraints6);
		}
		return topPanel;
	}

	/**
	 * This method initializes mainPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getMainPanel() {
		if (mainPanel == null) {
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.fill = GridBagConstraints.BOTH;
			gridBagConstraints9.gridy = 0;
			gridBagConstraints9.weightx = 1.0;
			gridBagConstraints9.weighty = 1.0;
			gridBagConstraints9.insets = new Insets(10, 10, 10, 10);
			gridBagConstraints9.ipady = 10;
			gridBagConstraints9.ipadx = 10;
			gridBagConstraints9.gridx = 5;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.anchor = GridBagConstraints.NORTHWEST;
			gridBagConstraints1.insets = new Insets(40, 20, 0, 0);
			gridBagConstraints1.gridwidth = 5;
			gridBagConstraints1.gridx = -1;
			gridBagConstraints1.gridy = -1;
			gridBagConstraints1.weighty = 1.0;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			mainPanel = new JPanel();
			mainPanel.setLayout(new GridBagLayout());
			mainPanel.setBorder(BorderFactory.createMatteBorder(0, 1, 0, 1, Color.black));
			mainPanel.add(mainText, gridBagConstraints1);
			mainPanel.add(getFileList(getFileListModel()), gridBagConstraints9);
		}
		return mainPanel;
	}

	/**
	 * This method initializes bottomPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getBottomPanel() {
		if (bottomPanel == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 4;
			gridBagConstraints11.insets = new Insets(5, 0, 5, 20);
			gridBagConstraints11.gridy = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.anchor = GridBagConstraints.CENTER;
			gridBagConstraints5.gridx = -1;
			gridBagConstraints5.gridy = -1;
			gridBagConstraints5.weightx = 1.0;
			gridBagConstraints5.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints5.insets = new Insets(5, 20, 5, 20);
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.anchor = GridBagConstraints.CENTER;
			gridBagConstraints2.gridx = -1;
			gridBagConstraints2.gridy = -1;
			gridBagConstraints2.ipady = 0;
			gridBagConstraints2.ipadx = 0;
			gridBagConstraints2.insets = new Insets(5, 0, 5, 20);
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.anchor = GridBagConstraints.CENTER;
			gridBagConstraints4.gridx = -1;
			gridBagConstraints4.gridy = -1;
			gridBagConstraints4.ipady = 0;
			gridBagConstraints4.insets = new Insets(5, 0, 5, 20);
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.anchor = GridBagConstraints.CENTER;
			gridBagConstraints.gridheight = 1;
			gridBagConstraints.gridx = -1;
			gridBagConstraints.gridy = -1;
			gridBagConstraints.ipadx = 0;
			gridBagConstraints.weightx = 0.0;
			gridBagConstraints.weighty = 0.0;
			gridBagConstraints.insets = new Insets(5, 0, 5, 20);
			bottomPanel = new JPanel();
			bottomPanel.setLayout(new GridBagLayout());
			bottomPanel.setBorder(BorderFactory.createLineBorder(Color.black, 1));
			bottomPanel.add(getUploadProgressBar(), gridBagConstraints5);
			bottomPanel.add(getConnectButton(), gridBagConstraints4);
			bottomPanel.add(getChooseFileButton(), gridBagConstraints);
			bottomPanel.add(getUploadButton(), gridBagConstraints2);
			bottomPanel.add(getDownloadButton(), gridBagConstraints11);
		}
		return bottomPanel;
	}

	/**
	 * This method initializes fileListModel	
	 * 	
	 * @return javax.swing.DefaultListModel	
	 */
	private DefaultListModel getFileListModel() {
		if (fileListModel == null) {
			fileListModel = new DefaultListModel();
		}
		return fileListModel;
	}

	/**
	 * This method initializes fileList	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getFileList(DefaultListModel dlm) {
		if (fileList == null) {
			fileList = new JList(dlm);
			fileList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			fileList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
					getDownloadButton().setEnabled(true);
					selectedFile = e.getLastIndex();
				}
			});
			fileList.setLayoutOrientation(JList.HORIZONTAL_WRAP);
		}
		return fileList;
	}

	/**
	 * This method initializes downloadButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getDownloadButton() {
		if (downloadButton == null) {
			downloadButton = new JButton();
			downloadButton.setEnabled(false);
			downloadButton.setText("Download");
			downloadButton.setActionCommand("download");
			downloadButton.addActionListener(new ActionHandler());
		}
		return downloadButton;
	}

}  //  @jve:decl-index=0:visual-constraint="-5,5"
