package server;

import java.io.IOException;
import java.util.Vector;

import server.data.FileElement;
import server.threads.MulticastListenerHandlerThread;
import server.threads.TCPDownloadListenerThread;
import server.threads.TCPUploadListenerThread;
import server.threads.UDPUploadListenerThread;
import auxdata.ConInfo;


public class ServerMain {
	
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		
		System.out.println("First ip: " +
				ConInfo.getTheTwoOtherServersIpAdresses()[0] + 
				"\nSecond ip: " + 
				ConInfo.getTheTwoOtherServersIpAdresses()[1]);

		// We need an indexed expandable datastructure.
		// Vector is thread safe, ArrayList is not.
		Vector<FileElement> fileStore = new Vector<FileElement>();
				
		// Listen for incoming Multicasts
		MulticastListenerHandlerThread mcht = new MulticastListenerHandlerThread(fileStore);
		mcht.start();
		// Listen for incoming TCP file upload requests
		TCPUploadListenerThread tcpult = new TCPUploadListenerThread(fileStore);
		tcpult.start();
		// Listen for incoming UDP file uploads
		UDPUploadListenerThread udpult = new UDPUploadListenerThread(fileStore);
		udpult.start();
		// Listen for incoming TCP file download requests
		TCPDownloadListenerThread tcpdlt = new TCPDownloadListenerThread(fileStore);
		tcpdlt.start();

		
		try {
			mcht.join();
			tcpult.join();
			udpult.join();
			tcpdlt.join();
		} catch (InterruptedException e) {
			System.err.println("Failed joining threads in ServerMain");
		}		

	}

}
