package server.data;


public class FileElement {
	
	public static final int FIRSTPART = 0;
	public static final int LASTPART = 1;
	public static final int XORSUM = 2;
	
	// In order
	
	private int ticketNumber = -1; // 4 bytes
	
	private int type; // 4 bytes
	
	private boolean evenSized; // 1 byte
	
	private byte[] filedata; // X bytes
	
	//
	
	
	private int offset = 9;
	private int arraySize = 0;
	
	
	/**
	 * @param ticketNumber / unique identifier
	 * @param type / first half(0), second half(1) or checksum(2)
	 * @param evenSized / whether the filesize is even or not
	 * @param filedata / self explanatory
	 */
	public FileElement (int ticketNumber, int type, boolean evenSized, byte[] filedata) {
		this.ticketNumber = ticketNumber;
		this.type = type;
		this.evenSized = evenSized;
		this.filedata = filedata;
		
		offset += filedata.length;
		
		arraySize = filedata.length + offset;
		
	}
	
	/**
	 * @param b /
	 * 
	 * Expect bytes to be ordered accordingly:
	 * 
	 * Bytes 1-4 = Ticketnumber
	 * Bytes 5-8 = FilePartType
	 * Byte 9 = Whether file is even size
	 * Bytes 10-x = Filedata
	 */
	public FileElement (byte[] b) {
		ticketNumber = UtilMethods.bytesToInt(b, 0);
		type = UtilMethods.bytesToInt(b, 4);
		evenSized = UtilMethods.byteToBoolean(b[9]);
		filedata = new byte[b.length - offset];
		
		
		for (int i = 0; i < filedata.length; i++) {
			filedata[i] = b[i + offset];
		}
		
		arraySize = b.length - offset;
		
	}
	
	/**
	 * @return /
	 * 
	 * Turns the file element into a byte array, 
	 * ready for transfer over UDP.
	 */
	public byte[] getBytes() {
		
		byte[] result = new byte[arraySize]; 
		
		// first four bytes is the ticketnumber
		byte[] tn = UtilMethods.intToBytes(ticketNumber);
		result[0] = tn[0];
		result[1] = tn[1];
		result[2] = tn[2];
		result[3] = tn[3];
		
		// next four bytes is the file part type
		byte[] tp = UtilMethods.intToBytes(type);
		result[4] = tp[0];
		result[5] = tp[1];
		result[6] = tp[2];
		result[7] = tp[3];
		
		// byte 9 is whether filesize is even
		result[8] = UtilMethods.booleanToByte(evenSized);
		
		// the rest is filedata
		for (int i = 0; i < filedata.length; i++) {
			result[i + offset] = filedata[i];
		}
		
		return result;
	}
	
	
//	
//	public byte [] getData () {
//		return filedata;
//	}
//	
//	public Boolean getEvenSized () {
//		return evenSized;
//	}
//	
	public int getTicketNumber () {
		return ticketNumber;
	}
//	
//	public Type getType () {
//		return type;
//	}

}
