package server.threads;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.util.Vector;

import server.data.FileElement;
import server.data.UtilMethods;
import auxdata.ConInfo;


/**
 * @author rene
 * 
 * This class listens for incoming broadcast requests for file upload.
 * 
 * It returns a simple acknowledgement to the requester, to make aware
 * of the servers presence on the network.
 *
 */
public class MulticastListenerHandlerThread extends Thread {
	
	private Vector<FileElement> fileStore;

	public MulticastListenerHandlerThread (Vector<FileElement> fileStore) {
		super("TCPHandlerThread");
		System.out.println("MultiCastThread running!");
		this.fileStore = fileStore;
	}

	public void run() {

		Boolean running = true;

		try {
			MulticastSocket socket = new MulticastSocket(ConInfo.MULTICASTPORT);
			InetAddress address = InetAddress.getByName(ConInfo.MULTICASTGROUP);
			socket.joinGroup(address);

			DatagramPacket packet;

			while (running) {
				byte[] buf = new byte[16];
				packet = new DatagramPacket(buf, buf.length);
				socket.receive(packet);

				String data = new String(packet.getData(), 0, packet.getLength());
				int possibleTicketNum = UtilMethods.bytesToInt(packet.getData(),0);
				
				// Look for possible ticket number in fileStore:
				boolean ticketFound = false;
				int i = 0;
				while (ticketFound == false && i < (fileStore.size() - 1)) {
					if (UtilMethods.bytesToInt(fileStore.get(i).getBytes(), 0) == possibleTicketNum) {
						ticketFound = true;
						int ticketSpot = i;
					}
					else
						i++;
				}

				// Client requested to upload new file
				if (data.equals(ConInfo.CLIENTRQUPLOAD)) {
					DatagramSocket returnSocket = new DatagramSocket();
					DatagramPacket returnPacket = 
						new DatagramPacket(
								ConInfo.ACK.getBytes(), ConInfo.ACK.length(),
								packet.getAddress(), packet.getPort());
					returnSocket.send(returnPacket);
				// Client has requested to download a file with a ticket number
				} else if (ticketFound && possibleTicketNum >= 0 && possibleTicketNum <= 5000) {
					DatagramSocket returnSocket = new DatagramSocket();
					DatagramPacket returnPacket = 
						new DatagramPacket(
								ConInfo.ACK.getBytes(), ConInfo.ACK.length(),
								packet.getAddress(), packet.getPort());
					returnSocket.send(returnPacket);
				// Another server wants to upload a file to this server
					
				} else if (data.equals(ConInfo.SERVERRQFILEUPLOAD)) {
					
					UDPUploadHandlerThread udpuht = new UDPUploadHandlerThread(packet.getAddress(), packet.getPort());
					udpuht.start();
				// Unrecognised request, resend data please
				} else { 
					DatagramSocket returnSocket = new DatagramSocket();
					DatagramPacket returnPacket = 
						new DatagramPacket(
								ConInfo.NAK.getBytes(), ConInfo.NAK.length(),
								packet.getAddress(), packet.getPort());
					returnSocket.send(returnPacket);
				}
			}

			socket.leaveGroup(address);
			socket.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
