package server.threads;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.ArrayList;

import server.data.UtilMethods;
import auxdata.ConInfo;

public class UDPUploadHandlerThread extends Thread {

	InetAddress clientAddress;
	int clientPort;

	public UDPUploadHandlerThread(InetAddress address, int port) {
		clientAddress = address;
		clientPort = port;
	}

	public void run () {
		
		DatagramSocket ds = null;
		DatagramPacket sendPacket;
		DatagramPacket receivePacket;		
		
		int fileSize;
		int blockCount;
		byte[] receiveBuffer;
		byte[] fileBuffer;
		byte[] file;
		
		try {
			
			 ds = new DatagramSocket(ConInfo.UDPRECEIVEPORT);

			// let client know we are ready to begin receiving
			sendPacket = new DatagramPacket(
					ConInfo.SERVERRQFILEUPLOADOK.getBytes(), ConInfo.SERVERRQFILEUPLOADOK.length(),
					clientAddress, clientPort);
			ds.send(sendPacket);
			
			// wait for the file size (assumption - no check for correctness)
			byte[] bytedFileSize = new byte[4];
			receivePacket = new DatagramPacket(bytedFileSize, bytedFileSize.length);
			ds.receive(receivePacket);
			
			// allocate space for the file
			fileSize = UtilMethods.bytesToInt(receivePacket.getData(), 0);
			blockCount = UtilMethods.getBlocksNeeded(fileSize);
			fileBuffer = new byte[blockCount * ConInfo.UDPPACKETDATASIZE];
			
			// send back ok for filesize
			sendPacket = new DatagramPacket(
					ConInfo.FILESIZERECEIVEDOK.getBytes(), 
					ConInfo.FILESIZERECEIVEDOK.getBytes().length,
					clientAddress, clientPort);
			ds.send(sendPacket);
			
			// make a list of blocks to be received
			ArrayList<String> blockTracker = new ArrayList<String>(blockCount);
			for (int i = 0; i < blockCount; i++) {
				blockTracker.add(i, "notReceived");
			}			
			
			// wait for filedata
			receiveBuffer = new byte[ConInfo.UDPPACKETSIZE];
			int seqNum;
			do {
				receivePacket = new DatagramPacket(
						receiveBuffer, receiveBuffer.length);
				ds.receive(receivePacket);
				
				seqNum = UtilMethods.bytesToInt(receivePacket.getData(), 0);
				if (seqNum > 0) {
					blockTracker.set(seqNum, "received");
					// copy file data to the filebuffer, 1020 bytes
					UtilMethods.copyArray(
							receivePacket.getData(), ConInfo.UDPPACKETHEADERLENGTH,
							fileBuffer, seqNum * ConInfo.UDPPACKETDATASIZE, ConInfo.UDPPACKETDATASIZE);
				}
				
				
			}
			while (seqNum > 0);
			
			// whole file received, look for missing fragments
			int index;
			byte[] bytedIndex;
			while (blockTracker.contains("notReceived")) {
				index = blockTracker.indexOf("notReceived");
				bytedIndex = UtilMethods.intToBytes(index);
				
				// request a resend of packet with seqNum index
				sendPacket = new DatagramPacket(
						bytedIndex, bytedIndex.length,
						clientAddress, clientPort);
				ds.send(sendPacket);
				
				// receive the packet
				receivePacket = new DatagramPacket(
						receiveBuffer, receiveBuffer.length);
				ds.receive(receivePacket);
				
				// this could loop infinetely
				seqNum = UtilMethods.bytesToInt(receivePacket.getData(), 0);
				if (seqNum > 0) {
					blockTracker.set(seqNum, "received");
					// copy file data to the filebuffer, 1020 bytes
					UtilMethods.copyArray(
							receivePacket.getData(), ConInfo.UDPPACKETHEADERLENGTH,
							fileBuffer, seqNum * ConInfo.UDPPACKETDATASIZE, ConInfo.UDPPACKETDATASIZE);
					// send back confirmation
					sendPacket = new DatagramPacket(
							ConInfo.ACK.getBytes(), ConInfo.ACK.getBytes().length,
							clientAddress, clientPort);
					ds.send(sendPacket);
				}
				
			}
			
			// all have been received, send back a negative seqNum to indicate EOF
			index = -1;
			bytedIndex = UtilMethods.intToBytes(index);
			sendPacket = new DatagramPacket(
					bytedIndex, bytedIndex.length,
					clientAddress, clientPort);
			ds.send(sendPacket);
			
			// all is done
			
			System.out.println("ajajaj");

		} catch (SocketException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		ds.close();

	}

}
